"""

formula:
chardev = (alphabet_letter - 1) * 32 + pin
"""
from string import ascii_uppercase


class Calculator:
    base = 32

    @staticmethod
    def __get_letter_index(letter):
        return [x for x in ascii_uppercase].index(letter) + 1

    @staticmethod
    def _get_pin_from_string(pin_string):
        """Retrieve the pin from a string.

        With a format like `H18` return the digits of it.

        Args:
            pin_string (str):

        Returns:
            (int) pin letter.
        """
        try:
            pin = int(pin_string[1:])

        except ValueError:
            raise ValueError(
                "Seems like {} cannot be translated into a pin.".format(pin_string[1:])
            )

        else:
            return pin

    @staticmethod
    def __get_letter_from_string(pin_string):
        """Retrieve the pin letter from a given string.

        Args:
            pin_string: string to get the letter from.

        Returns:
            (int) pin number.
        """
        return pin_string[0]

    def __guess_pin(self, chardev):
        """Guess the pin based on a chardev.

        Use the formula below in order to guess the pin
        looping through the letters.
        `pin = chardev - (32 * alphabet_letter) + 32`

        Returns:
            (list) possible_pins
        """
        possible_pins = list()

        for letter in ascii_uppercase:
            pin = chardev - (self.base * self.__get_letter_index(letter)) + self.base

            if self.get_chardev(letter, pin) == chardev and self.base >= pin >= 0:
                # Only append those that are below base and positive.
                possible_pins.append(pin)

        return possible_pins

    def get_pin(self, chardev):
        """Obtain a pin based on a chardev.

        Using the formula below obtain the pin number.
        `alphabet_letter = (chardev + 32 - pin) / 32`

        Returns:
            list with possible pins.
        """
        pins = self.__guess_pin(chardev)
        letters = [letter for letter in ascii_uppercase]

        combinations = list()

        for letter in letters:
            for pin in pins:

                if self.get_chardev(letter, pin) == chardev:
                    combinations.append("{letter}{pin}".format(letter=letter, pin=pin))

        if len(combinations) == 1:
            return combinations[0]

        else:
            return combinations

    def get_pins(self, input):
        """Get pins from list of chardev.

        Args:
            input (list): chardev to be converted.

        Returns:
            (dict) chardev: pin
        """
        pins = dict()

        for chardev in input:
            pins[chardev] = self.get_pin(chardev=chardev)

        return pins

    def get_chardev(self, letter, pin):
        """Calculate the chardev from a given letter and pin.

        Use the formula below to get the chardev with given pin and letter.
        `chardev = (alphabet_letter - 1) * 32 + pin`

        Returns:
            (int) chardev.
        """
        if letter.islower():
            letter = letter.upper()

        chardev = (self.__get_letter_index(letter) - 1) * self.base + pin

        return chardev

    def get_chardevs(self, input):
        """Get the chardev of given list of pins.

        Args:
            input (list): pins to get the chardev of.

        Returns:
            (dict) with the calculated chardevs.
        """

        chardevs = dict()

        for item in input:
            letter = self.__get_letter_from_string(item)
            pin = self._get_pin_from_string(item)
            chardevs[item] = self.get_chardev(letter, pin)

        return chardevs



"""
Unitary testing for gpio_calculator.calculator

"""

import logging
from unittest import mock, TestCase

from gpio_calculator.calculator import Calculator


class TestCalculator(TestCase):

    def setUp(self):
        """
        Test setUp
        """
        self.calculator = Calculator()

    def test_get_pin(self):
        """Test the `Calculator.get_pin` method."""
        expected = "H18"
        obtained = self.calculator.get_pin(242)

        self.assertEqual(expected, obtained, msg="Expected: {}, Obtained: {}".format(expected, obtained))

    def test_get_pins(self):
        """Test the `Calculator.get_pins` method."""
        expected = {
            242: "H18",
            243: "H19"
        }
        obtained = self.calculator.get_pins([242, 243])

        self.assertEqual(expected, obtained, msg="Expected: {}, Obtained: {}".format(expected, obtained))

    def test_get_chardev(self):
        """Test the `Calculator.get_chardev` method."""
        expected = 242
        obtained = self.calculator.get_chardev("H", 18)

        self.assertEqual(expected, obtained, msg="Expected: {}, Obtained: {}".format(expected, obtained))

    def test_get_chardev_lowercase(self):
        """Test the `Calculator.get_chardev` method.

        Check that is converts the letter into uppercase.
        """
        expected = 242
        obtained = self.calculator.get_chardev("h", 18)

        self.assertEqual(expected, obtained, msg="Expected: {}, Obtained: {}".format(expected, obtained))

    def test_get_chardevs(self):
        """Test the `Calculator.get_chardevs` method."""
        expected = {
            "H18": 242,
            "H19": 243
        }
        obtained = self.calculator.get_chardevs(["H18", "H19"])

        self.assertEqual(expected, obtained, msg="Expected: {}, Obtained: {}".format(expected, obtained))

    def test_get_pin_from_string(self):
        """Test the `Calculator._get_pin_from_string` method."""

        expected = 18
        obtained = self.calculator._get_pin_from_string("H18")

        self.assertEqual(expected, obtained, msg="Expected: {}, Obtained: {}".format(expected, obtained))

    def test_get_pin_from_string_abnormal(self):
        """Test the `Calculator._get_pin_from_string` method.

        Abnormal case when value is not possible to be converted.
        """

        with self.assertRaises(ValueError):
            self.calculator._get_pin_from_string("HAX")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

setup(
    name='GPIO-Calculator',
    version='1.0.2',
    use_scm_version=False,
    setup_requires=["setuptools_scm"],
    description='Simple GPIO calculator for libgpiod based on its chardev.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Juan Biondi',
    author_email='juanernestobiondi@gmail.com',
    python_requires='>=3.4.0',
    url='https://github.com/yeyeto2788/GPIO_Calculator',
    packages=['gpio_calculator', 'gpio_calculator.calculator'],
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],
)
